note
	description: "A correct binary search implementation with an explicit precondition of the input array being sorted."
	author: "Jackie Wang"
	date: "$Date$"
	revision: "$Revision$"

class
	UTILITIES_V3

create
	default_create

feature -- Binary Search

	search (a: ARRAY[STRING]; a_name: STRING): BOOLEAN
		require
			-- With the appropriate implementation of `{UTILITIES}.add_name` in effect,
			-- this precondition is never violated even if the user adds into the database in alphabetical order.

			-- Because of the addition of this precondition in V2, DATABASE_V3 is forced to correctly implement `add_name`
			-- such that the `data` array remains sorted.
			-- Therefore, DbC (e.g., addition of an appropriate precondition) forces a proper interaction
			-- between {DATABASE}.add_name and {UTILITIES}.search.

			-- When the current correct version is finalized, we may turn off the contract checking for performance,
			-- given that no contract violation would occur.

			array_sorted:
				across a.lower |..| (a.upper - 1) is i
					all a[i] < a[i + 1]
				end
		do
			Result := search_helper (a, a.lower, a.upper, a_name)
		end

feature {NONE} -- Recursive Helper

	search_helper (a: ARRAY[STRING]; left: INTEGER; right: INTEGER; a_name: STRING): BOOLEAN
		local
			mid: INTEGER
		do
			if left > right then
				Result := False
			elseif left = right then
				if a[left] ~ a_name then
					Result := True
				else
					Result := False
				end
			else
				mid := (left + right) // 2
				if a_name > a[mid] then
					Result := search_helper (a, mid + 1, right, a_name)
				elseif a_name < a[mid] then
					Result := search_helper (a, left, mid - 1, a_name)
				else
					Result := True
				end
			end
		end

end
